﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.ComponentModel;

namespace Obsolete
{
    public class CRMPresentation : INotifyPropertyChanged
    {
        public event PropertyChangedEventHandler PropertyChanged;

        private CRMData _crmData;
        private CustomerSelector _customerSelector;
        private Customer _currentCustomer;
        private string _mailMerge;

        public CRMPresentation(CRMData crmData, CustomerSelector customerSelector)
        {
            _crmData = crmData;
            _customerSelector = customerSelector;
            UpdateCurrentCustomer();

            // Register for changes to the selected customer.
            _customerSelector.PropertyChanged +=
                new PropertyChangedEventHandler(
                    CustomerSelector_PropertyChanged);
        }

        public CRMData CRMData
        {
            get { return _crmData; }
        }

        public CustomerSelector CustomerSelector
        {
            get { return _customerSelector; }
        }

        public string MailMerge
        {
            get { return _mailMerge; }
        }

        private void CustomerSelector_PropertyChanged(
            object sender, PropertyChangedEventArgs e)
        {
            if (e.PropertyName == "CurrentCustomer")
            {
                UpdateCurrentCustomer();
            }
        }

        void CurrentCustomer_PropertyChanged(
            object sender, PropertyChangedEventArgs e)
        {
            if (e.PropertyName == "Name")
            {
                UpdateMailMerge();
            }
        }

        private void UpdateCurrentCustomer()
        {
            // Unregister from the old customer.
            if (_currentCustomer != null)
                _currentCustomer.PropertyChanged -=
                    new PropertyChangedEventHandler(
                        CurrentCustomer_PropertyChanged);

            // Make the switch.
            _currentCustomer = _customerSelector.CurrentCustomer;
            UpdateMailMerge();

            // Register with the new customer.
            if (_currentCustomer != null)
                _currentCustomer.PropertyChanged +=
                    new PropertyChangedEventHandler(
                        CurrentCustomer_PropertyChanged);
        }

        private void UpdateMailMerge()
        {
            // Calculate the new mail merge.
            Customer recipient = _currentCustomer;
            if (recipient == null)
            {
                _mailMerge = string.Empty;
            }
            else
            {
                string name = recipient.Name;

                _mailMerge = string.Format(
                    "Congratulations {0}, you are a winner!",
                    name);
            }

            // Notify others that the mail merge has changed.
            FirePropertyChanged("MailMerge");
        }

        private void FirePropertyChanged(string name)
        {
            if (PropertyChanged != null)
                PropertyChanged.Invoke(this,
                    new PropertyChangedEventArgs(name));
        }

        public void NewCustomer()
        {
            Customer newCustomer = new Customer();
            _crmData.Customers.Add(newCustomer);
            _customerSelector.CurrentCustomer = newCustomer;
        }
    }
}
