﻿Imports System.ComponentModel

Public Class CRMPresentation
    Implements INotifyPropertyChanged

    Public Event PropertyChanged As PropertyChangedEventHandler Implements INotifyPropertyChanged.PropertyChanged

    Private _crmData As CRMData
    Private _customerSelector As CustomerSelector
    Private _currentCustomer As Customer
    Private _mailMerge As String

    Public Sub New(ByVal crmData As CRMData, ByVal customerSelector As CustomerSelector)
        _crmData = crmData
        _customerSelector = customerSelector
        UpdateCurrentCustomer()

        ' Register for changes to the selected customer.
        AddHandler _customerSelector.PropertyChanged, AddressOf Me.CustomerSelector_PropertyChanged
    End Sub

    Public ReadOnly Property CRMData() As CRMData
        Get
            Return _crmData
        End Get
    End Property

    Public ReadOnly Property CustomerSelector() As CustomerSelector
        Get
            Return _customerSelector
        End Get
    End Property

    Public ReadOnly Property MailMerge() As String
        Get
            Return _mailMerge
        End Get
    End Property

    Private Sub CustomerSelector_PropertyChanged(ByVal sender As Object, ByVal e As PropertyChangedEventArgs)
        If e.PropertyName = "CurrentCustomer" Then
            UpdateCurrentCustomer()
        End If
    End Sub

    Private Sub CurrentCustomer_PropertyChanged(ByVal sender As Object, ByVal e As PropertyChangedEventArgs)
        If e.PropertyName = "Name" Then
            UpdateMailMerge()
        End If
    End Sub

    Private Sub UpdateCurrentCustomer()
        ' Unregister from the old customer.
        If _currentCustomer IsNot Nothing Then
            RemoveHandler _currentCustomer.PropertyChanged, AddressOf CurrentCustomer_PropertyChanged
        End If

        ' Make the switch.
        _currentCustomer = _customerSelector.CurrentCustomer
        UpdateMailMerge()

        ' Register with the new customer.
        If _currentCustomer IsNot Nothing Then
            AddHandler _currentCustomer.PropertyChanged, AddressOf CurrentCustomer_PropertyChanged
        End If
    End Sub

    Private Sub UpdateMailMerge()
        ' Calculate the new mail merge.
        Dim recipient As Customer
        recipient = _currentCustomer
        If recipient Is Nothing Then
            _mailMerge = String.Empty
        Else
            _mailMerge = String.Format("Congratulations {0}, you are a winner!", recipient.Name)
        End If

        ' Notify others that mail merge has changed.
        FirePropertyChanged("MailMerge")
    End Sub

    Private Sub FirePropertyChanged(ByVal name As String)
        RaiseEvent PropertyChanged(Me, New PropertyChangedEventArgs(name))
    End Sub

    Public Sub NewCustomer()
        Dim newCustomer As New Customer

        _crmData.Customers.Add(newCustomer)
        _customerSelector.CurrentCustomer = newCustomer
    End Sub
End Class
